'' =================================================================================================
''
''   File....... jm_ws2812_seahawks.spin
''   Purpose.... WS2812/B demonstration program
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014-15 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 26 JAN 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us

  
                                                                  
con { io pins }                                                   
                                                                  
  RX1  = 31                                                      ' programming / terminal
  TX1  = 30                                                       
                                                                  
  SDA  = 29                                                      ' eeprom / i2c
  SCL  = 28                                                       
                                                                  
  LEDS = 15                                                      ' LED tx pin


con

  STRIP_LEN = 21
            
  
obj

  strip : "jm_ws2812"                                            ' WS2812 LED driver


var



pub main | pos

  strip.start_b(LEDS, STRIP_LEN, true)                           ' start led driver
  strip.off

  repeat
    repeat 20
      color_wipe($00_20_00, 10)
      color_wipe($00_00_20, 10)
    strip.off
    pause(100)

    chaser($00_00_20, $00_20_00, 167, 20)     
    strip.off
    pause(100)
                                              
          
pub chaser(color1, color2, ms, count) | ch

  repeat count
    ch := 0
    repeat while (ch < strip.num_pixels)                         ' first
      strip.set(ch++, color2)
      strip.set(ch++, color1)
      strip.set(ch++, color1)
    pause(ms)
    ch := 0
    repeat while (ch < strip.num_pixels)                         ' second
      strip.set(ch++, color1)
      strip.set(ch++, color2)
      strip.set(ch++, color1)
    pause(ms)
    ch := 0
    repeat while (ch < strip.num_pixels)                         ' third
      strip.set(ch++, color1)
      strip.set(ch++, color1)
      strip.set(ch++, color2)
    pause(ms)
    
                                                        
con

  ' Routines ported from C code by Phil Burgess (www.paintyourdragon.com)


pub color_wipe(rgb, ms) | ch

'' Sequentially fills strip with color rgb
'' -- ms is delay between pixels, in milliseconds

  repeat ch from 0 to strip.num_pixels-1 
    strip.set(ch, rgb)
    pause(ms)


pub rainbow(ms) | pos, ch

  repeat pos from 0 to 255
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, strip.wheel((pos + ch) & $FF))
    pause(ms)
    

pub rainbow_cycle(ms) | pos, ch 

  repeat pos from 0 to (255 * 5)
    repeat ch from 0 to strip.num_pixels-1
      strip.set(ch, strip.wheel(((ch * 256 / strip.num_pixels) + pos) & $FF))
    pause(ms)
    

con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds
                                                                  
  if (ms < 1)                                                    ' delay must be > 0
    return                                                        
  else                                                            
    t := cnt - 1776                                              ' sync with system counter
    repeat ms                                                    ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
 

dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}